@@ -3,11 +3,18 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import requests
+from bag.check_rst import check_rst_file
 from bag.console import bool_input
 from nine import input, nine
+from path import path as pathpy
 from . import ReleaseStep, StopRelease, CommandStep
 from .regex import version_in_python_source_file
 
+__all__ = ('Shell', 'CheckRstFiles', 'InteractivelyApproveDistribution',
+     'InteractivelyEnsureChangesDocumented', 'CheckTravis',
+    'SetVersionNumberInteractively', 'PypiRegister', 'PypiUpload',
+    'SetFutureVersion', 'ErrorStep')
+
 
 @nine
 class Shell(ReleaseStep):
@@ -26,19 +33,39 @@ class Shell(ReleaseStep):
         return '[' + self.COMMAND + ']'
 
 
+class CheckRstFiles(ReleaseStep):
+    '''Helps keep documentation correct by verifying .rst documentation files.
+    If files are not provided to the constructor, recursively finds *.rst files
+    in the current directory and subdirectories.
+    '''
+    ERROR_CODE = 4
+
+    def __init__(self, *files):
+        self.paths = files
+
+    def __call__(self):
+        paths = self.paths or pathpy('.').walkfiles('*.rst')
+        for path in paths:
+            self.log.info('Checking ' + path)
+            warnings = check_rst_file(path)
+            if warnings:
+                raise StopRelease('There are errors in {0}:\n{1}'.format(path,
+                    '\n'.join([str(w) for w in warnings])))
+
+
 class InteractivelyApproveDistribution(ReleaseStep):
     '''Generate a source distribution and let the user verify that
     all files are in there.
     '''
-    ERROR_CODE = 3
+    ERROR_CODE = 5
 
     def __call__(self):
         self._execute_or_complain('python setup.py sdist')
         # TODO: Optionally xdg-open the archive for convenience
         # Create the sdist with "-d <output_dir>"  on a temp dir.
         # Delete it at the end.
-        print("A source distribution has been generated. This is your chance ")
-        print("to open the archive (in the 'dist' directory) ")
+        print("A temporary source distribution has been generated. This is")
+        print("your chance to open the new archive (in the 'dist' directory)")
         print("and check that all files are in there.")
         if not bool_input("Do you approve the archive contents?"):
             raise StopRelease('Source distribution content not approved.\n'
@@ -48,6 +75,16 @@ class InteractivelyApproveDistribution(ReleaseStep):
                 "#specifying-the-files-to-distribute")
 
 
+class InteractivelyEnsureChangesDocumented(ReleaseStep):
+    ERROR_CODE = 3
+
+    def __call__(self):
+        if bool_input('Did you remember to update the CHANGES file?'):
+            self.log.debug('User says CHANGES file is up to date.')
+        else:
+            raise StopRelease('One more joshlabotniked release is avoided.')
+
+
 class CheckTravis(ReleaseStep):
     '''Checks the status, on travis-ci.org, of the latest build of a
     software project in a certain branch.
@@ -79,22 +116,25 @@ class SetVersionNumberInteractively(ReleaseStep):
     '''Asks the user for the new version number and writes it onto
     the configured source code file.
     '''
-    ERROR_CODE = 4
+    ERROR_CODE = 6
 
     def __call__(self):
         releaser = self.releaser
         path = releaser.config['version_file']
-        releaser.old_version = version_in_python_source_file(path)
+        keyword = releaser.config['version_keyword']
+        releaser.old_version = version_in_python_source_file(
+            path, keyword=keyword)
         print('Current version: {0}'.format(releaser.old_version))
         releaser.the_version = input('What is the new version number? ')
         # Write the new version onto the source code
-        version_in_python_source_file(path, replace=releaser.the_version)
+        version_in_python_source_file(
+            path, keyword=keyword, replace=releaser.the_version)
         self._succeed()
 
 
 class PypiRegister(CommandStep):
     COMMAND = 'python setup.py register'
-    ERROR_CODE = 5
+    ERROR_CODE = 7
     no_rollback = 'The release has been created on http://pypi.python.org,\n' \
         'Unfortunately, the release must be removed manually.'
 
@@ -104,7 +144,7 @@ class PypiRegister(CommandStep):
 
 class PypiUpload(CommandStep):
     COMMAND = 'python setup.py sdist upload'
-    ERROR_CODE = 6
+    ERROR_CODE = 8
     no_rollback = 'Cannot roll back the sdist upload to http://pypi.python.org'
 
     def _validate_command_output(self, command_output):
@@ -115,18 +155,21 @@ class SetFutureVersion(ReleaseStep):
     '''Replaces the version number in the configured source code file with
     the future version number (the one ending in 'dev').
     '''
-    ERROR_CODE = 7
+    ERROR_CODE = 9
 
     def __call__(self):
         releaser = self.releaser
         path = releaser.config['version_file']
+        keyword = releaser.config['version_keyword']
         # If the SetVersionNumberInteractively step is disabled for debugging,
         # we can still execute the current step, by populating the_version:
         if releaser.the_version is None:
-            releaser._the_version = version_in_python_source_file(path)
+            releaser._the_version = version_in_python_source_file(
+                path, keyword=keyword)
         self.log.info('Ready for the next development cycle! Setting version '
               + releaser.future_version)
-        version_in_python_source_file(path, replace=releaser.future_version)
+        version_in_python_source_file(
+            path, keyword=keyword, replace=releaser.future_version)
         self._succeed()
 
 
